@@ -35,24 +35,35 @@
 #ifndef CRYPTO_H
 #define CRYPTO_H
 
+#include "sqliteInt.h"
+#include "btreeInt.h"
+#include "pager.h"
+
+/* extensions defined in pager.c */ 
+void *sqlite3PagerGetCodec(Pager*);
+void sqlite3PagerSetCodec(Pager*, void *(*)(void*,void*,Pgno,int),  void (*)(void*,int,int),  void (*)(void*), void *);
+int sqlite3pager_is_mj_pgno(Pager*, Pgno);
+void sqlite3pager_error(Pager*, int);
+void sqlite3pager_reset(Pager *pPager);
+
 #if !defined (SQLCIPHER_CRYPTO_CC) \
    && !defined (SQLCIPHER_CRYPTO_LIBTOMCRYPT) \
+   && !defined (SQLCIPHER_CRYPTO_NSS) \
    && !defined (SQLCIPHER_CRYPTO_OPENSSL)
 #define SQLCIPHER_CRYPTO_OPENSSL
 #endif
 
 #define FILE_HEADER_SZ 16
 
-#ifndef CIPHER_VERSION
-#ifdef SQLCIPHER_FIPS
-#define CIPHER_VERSION "3.4.1 FIPS"
-#else
-#define CIPHER_VERSION "3.4.1"
-#endif
+#define CIPHER_XSTR(s) CIPHER_STR(s)
+#define CIPHER_STR(s) #s
+
+#ifndef CIPHER_VERSION_NUMBER
+#define CIPHER_VERSION_NUMBER 4.4.2
 #endif
 
-#ifndef CIPHER
-#define CIPHER "aes-256-cbc"
+#ifndef CIPHER_VERSION_BUILD
+#define CIPHER_VERSION_BUILD community
 #endif
 
 #define CIPHER_DECRYPT 0
@@ -63,7 +74,7 @@
 #define CIPHER_READWRITE_CTX 2
 
 #ifndef PBKDF2_ITER
-#define PBKDF2_ITER 64000
+#define PBKDF2_ITER 256000
 #endif
 
 /* possible flags for cipher_ctx->flags */
@@ -100,11 +111,30 @@
 #define CIPHER_MAX_KEY_SZ 64
 #endif
 
+#ifdef __ANDROID__
+#include <android/log.h>
+#endif
 
 #ifdef CODEC_DEBUG
-#define CODEC_TRACE(X)  {printf X;fflush(stdout);}
+#ifdef __ANDROID__
+#define CODEC_TRACE(...) {__android_log_print(ANDROID_LOG_DEBUG, "sqlcipher", __VA_ARGS__);}
 #else
-#define CODEC_TRACE(X)
+#define CODEC_TRACE(...)  {fprintf(stderr, __VA_ARGS__);fflush(stderr);}
+#endif
+#else
+#define CODEC_TRACE(...)
+#endif
+
+#ifdef CODEC_DEBUG_MUTEX
+#define CODEC_TRACE_MUTEX(...)  CODEC_TRACE(__VA_ARGS__)
+#else
+#define CODEC_TRACE_MUTEX(...)
+#endif
+
+#ifdef CODEC_DEBUG_MEMORY
+#define CODEC_TRACE_MEMORY(...)  CODEC_TRACE(__VA_ARGS__)
+#else
+#define CODEC_TRACE_MEMORY(...)
 #endif
 
 #ifdef CODEC_DEBUG_PAGEDATA
@@ -123,18 +153,6 @@
 #define CODEC_HEXDUMP(DESC,BUFFER,LEN)
 #endif
 
-/* extensions defined in pager.c */ 
-void sqlite3pager_get_codec(Pager *pPager, void **ctx);
-int sqlite3pager_is_mj_pgno(Pager *pPager, Pgno pgno);
-sqlite3_file *sqlite3Pager_get_fd(Pager *pPager);
-void sqlite3pager_sqlite3PagerSetCodec(
-  Pager *pPager,
-  void *(*xCodec)(void*,void*,Pgno,int),
-  void (*xCodecSizeChng)(void*,int,int),
-  void (*xCodecFree)(void*),
-  void *pCodec
-);
-void sqlite3pager_sqlite3PagerSetError(Pager *pPager, int error);
 /* end extensions defined in pager.c */
  
 /*
@@ -174,12 +192,62 @@ static int cipher_isHex(const unsigned char *hex, int sz){
 }
 
 /* extensions defined in crypto_impl.c */
-typedef struct codec_ctx codec_ctx;
+/* the default implementation of SQLCipher uses a cipher_ctx
+   to keep track of read / write state separately. The following
+   struct and associated functions are defined here */
+typedef struct {
+  int derive_key;
+  int pass_sz;
+  unsigned char *key;
+  unsigned char *hmac_key;
+  unsigned char *pass;
+  char *keyspec;
+} cipher_ctx;
+
+
+typedef struct {
+  int store_pass;
+  int kdf_iter;
+  int fast_kdf_iter;
+  int kdf_salt_sz;
+  int key_sz;
+  int iv_sz;
+  int block_sz;
+  int page_sz;
+  int keyspec_sz;
+  int reserve_sz;
+  int hmac_sz;
+  int plaintext_header_sz;
+  int hmac_algorithm;
+  int kdf_algorithm;
+  unsigned int skip_read_hmac;
+  unsigned int need_kdf_salt;
+  unsigned int flags;
+  unsigned char *kdf_salt;
+  unsigned char *hmac_kdf_salt;
+  unsigned char *buffer;
+  Btree *pBt;
+  cipher_ctx *read_ctx;
+  cipher_ctx *write_ctx;
+  sqlcipher_provider *provider;
+  void *provider_ctx;
+} codec_ctx ;
+
+/* crypto.c functions */
+int sqlcipher_codec_pragma(sqlite3*, int, Parse*, const char *, const char*);
+int sqlite3CodecAttach(sqlite3*, int, const void *, int);
+void sqlite3CodecGetKey(sqlite3*, int, void**, int*);
+void sqlcipher_exportFunc(sqlite3_context *, int, sqlite3_value **);
+
+/* crypto_impl.c functions */
+
+void sqlcipher_init_memmethods(void);
 
 /* activation and initialization */
-void sqlcipher_activate();
-void sqlcipher_deactivate();
-int sqlcipher_codec_ctx_init(codec_ctx **, Db *, Pager *, sqlite3_file *, const void *, int);
+void sqlcipher_activate(void);
+void sqlcipher_deactivate(void);
+
+int sqlcipher_codec_ctx_init(codec_ctx **, Db *, Pager *, const void *, int);
 void sqlcipher_codec_ctx_free(codec_ctx **);
 int sqlcipher_codec_key_derive(codec_ctx *);
 int sqlcipher_codec_key_copy(codec_ctx *, int);
@@ -190,6 +258,7 @@ int sqlcipher_page_cipher(codec_ctx *, int, Pgno, int, int, unsigned char *, uns
 /* context setters & getters */
 void sqlcipher_codec_ctx_set_error(codec_ctx *, int);
 
+void sqlcipher_codec_get_pass(codec_ctx *, void **, int *);
 int sqlcipher_codec_ctx_set_pass(codec_ctx *, const void *, int, int);
 void sqlcipher_codec_get_keyspec(codec_ctx *, void **zKey, int *nKey);
 
@@ -198,49 +267,68 @@ int sqlcipher_codec_ctx_get_pagesize(codec_ctx *);
 int sqlcipher_codec_ctx_get_reservesize(codec_ctx *);
 
 void sqlcipher_set_default_pagesize(int page_size);
-int sqlcipher_get_default_pagesize();
+int sqlcipher_get_default_pagesize(void);
 
 void sqlcipher_set_default_kdf_iter(int iter);
-int sqlcipher_get_default_kdf_iter();
+int sqlcipher_get_default_kdf_iter(void);
+int sqlcipher_codec_ctx_set_kdf_iter(codec_ctx *, int);
+int sqlcipher_codec_ctx_get_kdf_iter(codec_ctx *ctx);
 
-int sqlcipher_codec_ctx_set_kdf_iter(codec_ctx *, int, int);
-int sqlcipher_codec_ctx_get_kdf_iter(codec_ctx *ctx, int);
+int sqlcipher_codec_ctx_set_kdf_salt(codec_ctx *ctx, unsigned char *salt, int sz);
+int sqlcipher_codec_ctx_get_kdf_salt(codec_ctx *ctx, void **salt);
 
-void* sqlcipher_codec_ctx_get_kdf_salt(codec_ctx *ctx);
+int sqlcipher_codec_ctx_set_fast_kdf_iter(codec_ctx *, int);
+int sqlcipher_codec_ctx_get_fast_kdf_iter(codec_ctx *);
 
-int sqlcipher_codec_ctx_set_fast_kdf_iter(codec_ctx *, int, int);
-int sqlcipher_codec_ctx_get_fast_kdf_iter(codec_ctx *, int);
-
-int sqlcipher_codec_ctx_set_cipher(codec_ctx *, const char *, int);
-const char* sqlcipher_codec_ctx_get_cipher(codec_ctx *ctx, int for_ctx);
+const char* sqlcipher_codec_ctx_get_cipher(codec_ctx *ctx);
 
 void* sqlcipher_codec_ctx_get_data(codec_ctx *);
 
-void sqlcipher_exportFunc(sqlite3_context *, int, sqlite3_value **);
-
 void sqlcipher_set_default_use_hmac(int use);
-int sqlcipher_get_default_use_hmac();
+int sqlcipher_get_default_use_hmac(void);
 
 void sqlcipher_set_hmac_salt_mask(unsigned char mask);
-unsigned char sqlcipher_get_hmac_salt_mask();
+unsigned char sqlcipher_get_hmac_salt_mask(void);
 
 int sqlcipher_codec_ctx_set_use_hmac(codec_ctx *ctx, int use);
-int sqlcipher_codec_ctx_get_use_hmac(codec_ctx *ctx, int for_ctx);
+int sqlcipher_codec_ctx_get_use_hmac(codec_ctx *ctx);
 
 int sqlcipher_codec_ctx_set_flag(codec_ctx *ctx, unsigned int flag);
 int sqlcipher_codec_ctx_unset_flag(codec_ctx *ctx, unsigned int flag);
-int sqlcipher_codec_ctx_get_flag(codec_ctx *ctx, unsigned int flag, int for_ctx);
+int sqlcipher_codec_ctx_get_flag(codec_ctx *ctx, unsigned int flag);
 
 const char* sqlcipher_codec_get_cipher_provider(codec_ctx *ctx);
 int sqlcipher_codec_ctx_migrate(codec_ctx *ctx);
 int sqlcipher_codec_add_random(codec_ctx *ctx, const char *data, int random_sz);
 int sqlcipher_cipher_profile(sqlite3 *db, const char *destination);
-static void sqlcipher_profile_callback(void *file, const char *sql, sqlite3_uint64 run_time);
-static int sqlcipher_codec_get_store_pass(codec_ctx *ctx);
-static void sqlcipher_codec_get_pass(codec_ctx *ctx, void **zKey, int *nKey);
-static void sqlcipher_codec_set_store_pass(codec_ctx *ctx, int value);
+int sqlcipher_codec_get_store_pass(codec_ctx *ctx);
+void sqlcipher_codec_get_pass(codec_ctx *ctx, void **zKey, int *nKey);
+void sqlcipher_codec_set_store_pass(codec_ctx *ctx, int value);
 int sqlcipher_codec_fips_status(codec_ctx *ctx);
 const char* sqlcipher_codec_get_provider_version(codec_ctx *ctx);
+
+int sqlcipher_set_default_plaintext_header_size(int size);
+int sqlcipher_get_default_plaintext_header_size(void);
+int sqlcipher_codec_ctx_set_plaintext_header_size(codec_ctx *ctx, int size);
+int sqlcipher_codec_ctx_get_plaintext_header_size(codec_ctx *ctx);
+
+int sqlcipher_set_default_hmac_algorithm(int algorithm);
+int sqlcipher_get_default_hmac_algorithm(void);
+int sqlcipher_codec_ctx_set_hmac_algorithm(codec_ctx *ctx, int algorithm);
+int sqlcipher_codec_ctx_get_hmac_algorithm(codec_ctx *ctx);
+
+int sqlcipher_set_default_kdf_algorithm(int algorithm);
+int sqlcipher_get_default_kdf_algorithm(void);
+int sqlcipher_codec_ctx_set_kdf_algorithm(codec_ctx *ctx, int algorithm);
+int sqlcipher_codec_ctx_get_kdf_algorithm(codec_ctx *ctx);
+
+void sqlcipher_set_mem_security(int);
+int sqlcipher_get_mem_security(void);
+
+int sqlcipher_find_db_index(sqlite3 *db, const char *zDb);
+
+int sqlcipher_codec_ctx_integrity_check(codec_ctx *, Parse *, char *);
+
 #endif
 #endif
 /* END SQLCIPHER */
